// locale.ts
var LANGUAGES = [
  "ar",
  "de",
  "en",
  "es",
  "eu",
  "fi",
  "fr",
  "he",
  "ja",
  "nl",
  "pl",
  "pt",
  "to",
  "tr",
  "zh_Hans",
  "zh_Hant"
];
var TR = {};
async function init_locale(lang) {
  if (!lang) return await init_locale(select_language());
  const res = await fetch(`/locale/${encodeURIComponent(lang)}.json`, { headers: { "Accept": "application/json" } });
  if (!res.ok) throw new Error("language pack download failed");
  TR = await res.json();
}
function tr(key, ...args) {
  let s = TR[key];
  if (!s) return key;
  if (args.length) s = s.replace(/{(\d+)}/ig, (_m, index) => args[parseInt(index)]);
  return s;
}
function select_language() {
  const plang = new URLSearchParams(globalThis.location.hash.substring(1)).get("lang");
  if (plang) return plang;
  const navlang = navigator.language.split("-")[0] ?? "en";
  if (LANGUAGES.includes(navlang)) return navlang;
  console.warn("fallback language selected");
  return "en";
}

// util.ts
function length(p) {
  return Math.sqrt(p.x * p.x + p.y * p.y);
}
function normalize(p) {
  let l = length(p);
  if (l == 0) l = 1;
  return { x: p.x / l, y: p.y / l };
}
function lerp_exp(current, target, dt) {
  return target + (current - target) * Math.exp(-dt);
}
function lerp_exp_v2_mut(current, target, dt) {
  current.x = target.x + (current.x - target.x) * Math.exp(-dt);
  current.y = target.y + (current.y - target.y) * Math.exp(-dt);
}
function floor_v2(p) {
  return { x: Math.floor(p.x), y: Math.floor(p.y) };
}
function ceil_v2(p) {
  return { x: Math.ceil(p.x), y: Math.ceil(p.y) };
}
function sub_v2(p, o) {
  return { x: p.x - o.x, y: p.y - o.y };
}

// movement.ts
var PLAYER_SIZE = 0.4;
var PLAYER_FRICTION = 15;
var PLAYER_SPEED = 55;
var BOOST_FACTOR = 2.5;
var BOOST_DURATION = 0.3;
var BOOST_RESTORE = 0.5;
function update_movement(p, dt, direction, boost) {
  if (length(direction) > 0.05) lerp_exp_v2_mut(p.facing, direction, dt * 10);
  if (length(direction) < 0.5) direction.x = direction.y = 0;
  p.rot = Math.atan2(p.facing.x, p.facing.y);
  boost &&= length(direction) > 0.1;
  p.boosting = boost && (p.boosting || p.stamina >= 1) && p.stamina > 0;
  if (p.boosting) p.stamina -= dt / BOOST_DURATION;
  else p.stamina += dt / BOOST_RESTORE;
  p.stamina = Math.max(Math.min(p.stamina, 1), 0);
  const speed = PLAYER_SPEED * (p.boosting ? BOOST_FACTOR : 1);
  p.vel.x += direction.x * dt * speed;
  p.vel.y += direction.y * dt * speed;
  p.position.x += p.vel.x * dt;
  p.position.y += p.vel.y * dt;
  collide_player(p, dt);
  lerp_exp_v2_mut(p.vel, { x: 0, y: 0 }, dt * PLAYER_FRICTION);
}
function collide_player(p, _dt) {
  for (let xo = -1; xo <= 1; xo++) {
    for (let yo = -1; yo <= 1; yo++) {
      const x = Math.floor(p.position.x) + xo;
      const y = Math.floor(p.position.y) + yo;
      const tile = tiles.get([x, y].toString());
      if (tile && !data.tile_collide[tile.kind]) continue;
      const d = aabb_point_distance(x, y, x + 1, y + 1, p.position.x, p.position.y);
      if (d > PLAYER_SIZE) continue;
      const h = 0.01;
      const d_sample_x = aabb_point_distance(x, y, x + 1, y + 1, p.position.x + h, p.position.y);
      const d_sample_y = aabb_point_distance(x, y, x + 1, y + 1, p.position.x, p.position.y + h);
      const grad_x = (d_sample_x - d) / h;
      const grad_y = (d_sample_y - d) / h;
      p.position.x += (PLAYER_SIZE - d) * grad_x;
      p.position.y += (PLAYER_SIZE - d) * grad_y;
      const vdotn = grad_x * p.vel.x + grad_y * p.vel.y;
      p.vel.x -= grad_x * vdotn;
      p.vel.y -= grad_y * vdotn;
    }
  }
}
function collide_player_player(a, b, dt) {
  const diff = sub_v2(a.position, b.position);
  const d = length(diff);
  if (d < 0.01) return;
  if (d >= PLAYER_SIZE * 2) return;
  const norm = normalize(diff);
  const f = 100 / (1 + d);
  a.vel.x += norm.x * f * dt;
  a.vel.y += norm.y * f * dt;
}
function aabb_point_distance(min_x, min_y, max_x, max_y, px, py) {
  const dx = px - Math.max(min_x, Math.min(max_x, px));
  const dy = py - Math.max(min_y, Math.min(max_y, py));
  return Math.sqrt(dx * dx + dy * dy);
}

// particles.ts
var particles = /* @__PURE__ */ new Set();
var particles_to_remove = /* @__PURE__ */ new Set();
function tick_particles(dt) {
  particles.forEach((p) => p.tick(dt) ? {} : particles_to_remove.add(p));
  particles_to_remove.forEach((p) => particles.delete(p));
  particles_to_remove = /* @__PURE__ */ new Set();
}
function draw_particles() {
  particles.forEach((p) => p.draw());
}
function particle_count() {
  return particles.size;
}
function particle_splash(pos, hue) {
  for (let i = 0; i < 64; i++) {
    const p = new Particle();
    p.px = pos.x + 0.5;
    p.py = pos.y + 0.5;
    const a = Math.random() * Math.PI * 2;
    const r = Math.sqrt(Math.random()) * 5;
    p.vx = Math.sin(a) * r;
    p.vy = Math.cos(a) * r;
    p.decay = 4;
    p.tr = Math.sqrt(Math.random()) * 1;
    p.c = `hsl(${Math.random() * 0.2 - 0.1 + hue}turn, 100%, 50%)`;
    p.r = Math.random() * 0.1 + 0.03;
    particles.add(p);
  }
}
var Particle = class {
  c = "red";
  r = 0.05;
  tr = 0;
  px = 0;
  py = 0;
  vx = 0;
  vy = 0;
  decay = 1;
  tick(dt) {
    this.tr -= dt;
    this.px += this.vx * dt;
    this.py += this.vy * dt;
    this.vx *= Math.exp(-dt * this.decay);
    this.vy *= Math.exp(-dt * this.decay);
    return this.tr > 0;
  }
  draw() {
    ctx.fillStyle = this.c;
    ctx.globalAlpha = Math.min(1, this.tr * 5);
    ctx.beginPath();
    ctx.arc(this.px, this.py, this.r, 0, Math.PI * 2);
    ctx.fill();
  }
};

// tiles.ts
function base(fill, stroke, stroke_width) {
  return (c) => {
    c.fillStyle = fill;
    c.strokeStyle = stroke ?? "black";
    c.lineWidth = stroke_width ?? 0.05;
    c.lineJoin = "miter";
    c.lineCap = "square";
    c.fillRect(
      -0.5,
      -0.5,
      1,
      1
    );
    if (stroke) c.strokeRect(
      -0.5 + c.lineWidth / 2,
      -0.5 + c.lineWidth / 2,
      1 - c.lineWidth,
      1 - c.lineWidth
    );
  };
}
function rect(inset, fill, stroke, stroke_width) {
  return (c) => {
    c.fillStyle = fill;
    c.strokeStyle = stroke ?? "black";
    c.lineWidth = stroke_width ?? 0.05;
    c.lineJoin = "round";
    c.lineCap = "round";
    c.fillRect(
      -0.5 + inset,
      -0.5 + inset,
      1 - inset * 2,
      1 - inset * 2
    );
    if (stroke) c.strokeRect(
      -0.5 + inset,
      -0.5 + inset,
      1 - inset * 2,
      1 - inset * 2
    );
  };
}
function circle(radius, fill, stroke, stroke_width) {
  return (c) => {
    c.fillStyle = fill;
    c.strokeStyle = stroke ?? "black";
    c.lineWidth = stroke_width ?? 0.05;
    c.beginPath();
    c.arc(0, 0, radius, 0, Math.PI * 2);
    if (stroke) c.stroke();
    c.fill();
  };
}
function cross(size, stroke, stroke_width = 0.05) {
  return (c) => {
    c.strokeStyle = stroke;
    c.lineWidth = stroke_width;
    c.lineCap = "round";
    c.beginPath();
    c.moveTo(-size, -size);
    c.lineTo(size, size);
    c.moveTo(size, -size);
    c.lineTo(-size, size);
    c.stroke();
  };
}
function text(s) {
  return (c) => {
    c.font = "0.8px sans-serif";
    c.strokeStyle = "#e38242";
    c.fillStyle = "white";
    c.lineWidth = 0.05;
    c.textAlign = "center";
    c.textBaseline = "middle";
    c.lineJoin = "round";
    c.lineCap = "round";
    c.strokeText(s, 0, 0);
    c.fillText(s, 0, 0);
  };
}
function arrange_items(...items) {
  return (c) => {
    for (let index = 0; index < items.length; index++) {
      const item = items[index];
      const t = index / items.length * Math.PI * 2;
      const radius = items.length == 1 ? 0 : 0.4 / items.length;
      const off_x = Math.sin(t) * radius;
      const off_y = Math.cos(t) * radius;
      const scale = 1 / Math.sqrt(items.length);
      c.save();
      c.translate(off_x, off_y);
      c.scale(scale, scale);
      iref(item)(c);
      c.restore();
    }
  };
}
var door = (c) => {
  c.fillStyle = "#ff9843";
  c.fillRect(-0.5, -0.1, 1, 0.2);
};
var iref = (name) => (c) => draw_item_sprite(c, name);
var tref = (name) => (c) => TILES[name].forEach((f) => f(c));
var ITEMS = {
  "bun": () => circle(0.3, "#853e20"),
  "burned": () => (c) => (circle(0.3, "rgb(61, 18, 0)")(c), cross(0.2, "red", 0.02)(c)),
  "coconut": () => circle(0.3, "rgb(75, 49, 25)"),
  "cooked-rice": () => circle(0.3, "rgb(233, 233, 233)"),
  "curry": () => circle(0.3, "rgb(185, 67, 37)"),
  "dough": () => circle(0.3, "#b38d7d"),
  "fish": () => circle(0.3, "rgb(62, 66, 104)"),
  "flour": () => circle(0.3, "#d8c9c2"),
  "leek": () => circle(0.3, "rgb(50, 133, 17)"),
  "milk": () => circle(0.3, "rgb(252, 243, 208)"),
  "nigiri": () => circle(0.25, "rgb(233, 233, 233)", "salmon"),
  "rice": () => circle(0.3, "rgb(163, 163, 163)"),
  "seared-steak": () => circle(0.3, "#702200"),
  "sliced-bun": () => circle(0.3, "#853e20"),
  "sliced-fish": () => circle(0.3, "salmon", "rgb(62, 66, 104)"),
  "steak": () => circle(0.3, "#ca3510"),
  "patty": () => circle(0.3, "#c26149"),
  "seared-patty": () => circle(0.3, "#502c23"),
  "strawberry-icecream": () => circle(0.2, "rgb(250, 148, 236)"),
  "strawberry-mochi": () => circle(0.2, "rgb(161, 111, 132)"),
  "strawberry-shake": () => circle(0.3, "rgb(255, 180, 180)"),
  "strawberry": () => circle(0.3, "rgb(228, 79, 111)"),
  "tomato-juice": () => circle(0.3, "#b80000"),
  "tomato-soup": () => circle(0.3, "#ff2600"),
  "water": () => circle(0.3, "rgb(86, 92, 206)"),
  "tomato": () => circle(0.3, "#d63838"),
  "sliced-tomato": () => circle(0.3, "#d16363", "#d63838", 0.08),
  "lettuce": () => circle(0.3, "#64a30b"),
  "sliced-lettuce": () => circle(0.3, "#a0da4f", "#64a30b", 0.08),
  "cheese": () => circle(0.3, "#b3b615"),
  "sliced-cheese": () => rect(0.25, "#dcdf29", "#b3b615", 0.05),
  "dirty-plate": () => circle(0.4, "#947a6f", "#d3a187", 0.02),
  "mochi-dough": () => circle(0.3, "rgb(172, 162, 151)"),
  "rice-flour": () => iref("rice"),
  "unknown-order": () => text("!"),
  "pan": (i) => (c) => (circle(0.35, "rgb(29, 29, 29)", "rgb(39, 39, 39)", 0.04)(c), arrange_items(...i)(c)),
  "pot": (i) => (c) => (circle(0.27, "rgb(29, 29, 29)", "rgb(56, 56, 56)", 0.2)(c), arrange_items(...i)(c)),
  "foodprocessor": (i) => (c) => (circle(0.35, "rgb(86, 168, 189)", "rgb(88, 222, 255)", 0.04)(c), arrange_items(...i)(c)),
  "plate": (i) => (c) => (circle(0.4, "#b6b6b6", "#f7f7f7", 0.02)(c), arrange_items(...i)(c)),
  "glass": (i) => (c) => (circle(0.35, "rgb(150, 255, 237)", "rgb(52, 129, 155)", 0.02), arrange_items(...i)(c))
};
var crate = (i) => [base("#60701e", "#b9da37", 0.05), iref(i)];
var floor = tref("floor");
var counter = tref("counter");
var TILES = {
  "floor": [base("#333", "#222", 0.05)],
  "street": [base("rgb(19, 19, 19)")],
  "table": [base("rgb(133, 76, 38)")],
  "door": [floor, door],
  "chair": [floor, circle(0.45, "rgb(136, 83, 41)")],
  "wall": [base("rgb(0, 14, 56)")],
  "wall-window": [base("rgb(19, 40, 102)")],
  "counter": [base("rgb(182, 172, 164)")],
  "counter-window": [base("rgb(233, 233, 233)")],
  "grass": [base("rgb(0, 107, 4)")],
  "path": [base("rgb(100, 80, 55)")],
  "conveyor": [base("rgb(107, 62, 128)")],
  "tree": [base("rgb(1, 82, 4)")],
  "cuttingboard": [counter, rect(0.3, "rgb(158, 236, 68)", "rgb(158, 236, 68)", 0.2)],
  "trash": [floor, circle(0.4, "rgb(20, 20, 20)"), cross(0.3, "rgb(90, 36, 36)")],
  "sink": [base("rgb(131, 129, 161)", "rgb(177, 174, 226)", 0.2)],
  "oven": [base("rgb(241, 97, 61)", "rgb(109, 84, 84)", 0.3)],
  "freezer": [base("rgb(61, 97, 241)", "rgb(84, 88, 109)", 0.3)],
  "stove": [counter, circle(0.4, "#444", "#999")],
  "book": [counter, rect(0.2, "rgb(88, 44, 7)")],
  "lamp": [tref("grass"), rect(0.3, "rgb(255, 217, 127)", "rgb(32, 32, 32)", 0.1)],
  "flour-crate": crate("flour"),
  "steak-crate": crate("steak"),
  "tomato-crate": crate("tomato"),
  "lettuce-crate": crate("lettuce"),
  "cheese-crate": crate("cheese"),
  "leek-crate": crate("leek"),
  "rice-crate": crate("rice"),
  "fish-crate": crate("fish"),
  "coconut-crate": crate("coconut"),
  "strawberry-crate": crate("strawberry")
};
function debug_label(ctx2, name) {
  ctx2.save();
  ctx2.font = "10px sans-serif";
  ctx2.fillStyle = "white";
  ctx2.strokeStyle = "black";
  ctx2.lineWidth = 1;
  ctx2.textAlign = "center";
  ctx2.textBaseline = "middle";
  ctx2.scale(0.03, 0.03);
  ctx2.strokeText(name, 0, 0);
  ctx2.fillText(name, 0, 0);
  ctx2.restore();
}
function draw_item_sprite(ctx2, name) {
  const [base2, cont] = name.split(":");
  const c = ITEMS[base2];
  if (c) {
    c(cont?.split(",") ?? [])(ctx2);
  } else {
    circle(0.4, "#f0f")(ctx2);
    debug_label(ctx2, name);
  }
}
function draw_tile_sprite(ctx2, name) {
  const comps = TILES[name];
  if (comps) {
    for (const c of comps) {
      c(ctx2);
    }
  } else {
    base("#f0f")(ctx2);
    debug_label(ctx2, name);
  }
}

// visual.ts
function draw_wait(text2) {
  ctx.fillStyle = "#444";
  ctx.fillRect(0, 0, canvas.width, canvas.height);
  ctx.fillStyle = "#555";
  ctx.font = "50px sans-serif";
  ctx.strokeStyle = "black";
  ctx.fillStyle = "white";
  ctx.lineWidth = 10;
  ctx.textAlign = "center";
  ctx.textBaseline = "middle";
  ctx.lineJoin = "round";
  ctx.lineCap = "round";
  ctx.strokeText(text2, canvas.width / 2, canvas.height / 2);
  ctx.fillText(text2, canvas.width / 2, canvas.height / 2);
}
function draw_ingame() {
  ctx.fillStyle = "#111";
  ctx.fillRect(0, 0, canvas.width, canvas.height);
  ctx.save();
  ctx.translate(canvas.width / 2, canvas.height / 2);
  ctx.scale(camera_scale, camera_scale);
  ctx.translate(-camera.x, -camera.y);
  draw_grid();
  for (const [_, tile] of tiles)
    draw_tile(tile);
  for (const [_, player] of players)
    draw_player(player);
  for (const item of items_removed)
    draw_item(item);
  for (const [_, tile] of tiles)
    if (tile.item) draw_item(tile.item);
  draw_particles();
  draw_interact_target();
  for (const [_, player] of players)
    if (player.message) draw_message(player.message, true);
  for (const [_, player] of players)
    if (player.message_persist) draw_message(player.message_persist, true);
  for (const [_, player] of players)
    draw_player_nametag(player);
  for (const [_, message] of server_hints)
    draw_message(message, true);
  ctx.restore();
  ctx.save();
  ctx.translate(50, 50);
  ctx.scale(80, 80);
  for (const [_, player] of players)
    if (player.message_pinned) draw_message(player.message_pinned, false);
  ctx.restore();
  draw_global_message();
  if (!is_lobby)
    draw_score();
  if (keys_down.has("KeyP"))
    draw_debug();
}
function draw_score() {
  ctx.fillStyle = "white";
  ctx.textAlign = "left";
  ctx.textBaseline = "bottom";
  ctx.font = "20px sans-serif";
  ctx.fillText(`${tr("c.score.time_remaining")}: ${score.time_remaining?.toFixed(2)}`, 10, canvas.height - 90);
  ctx.font = "30px sans-serif";
  ctx.fillText(`${tr("c.score.points")}: ${score.points}`, 10, canvas.height - 60);
  ctx.font = "20px sans-serif";
  ctx.fillText(`${tr("c.score.completed")}: ${score.demands_completed}`, 10, canvas.height - 30);
  ctx.fillText(`${tr("c.score.failed")}: ${score.demands_failed}`, 10, canvas.height - 10);
}
function draw_debug() {
  ctx.fillStyle = "white";
  ctx.textAlign = "left";
  ctx.textBaseline = "bottom";
  ctx.font = "20px sans-serif";
  ctx.fillText(`position = ${JSON.stringify(players.get(my_id)?.anim_position)}`, 10, 30);
  ctx.fillText(`velocity = ${JSON.stringify(players.get(my_id)?.vel)}`, 10, 50);
  ctx.fillText(`interact = ${JSON.stringify(get_interact_target())}`, 10, 70);
  ctx.fillText(`particle_count = ${particle_count()}`, 10, 90);
}
function draw_tile(tile) {
  ctx.save();
  ctx.translate(tile.x + 0.5, tile.y + 0.5);
  draw_tile_sprite(ctx, data.tile_names[tile.kind]);
  ctx.restore();
}
function draw_item(item) {
  ctx.save();
  ctx.translate(item.x, item.y);
  if (item.remove_anim) ctx.scale(1 - item.remove_anim, 1 - item.remove_anim);
  draw_item_sprite(ctx, data.item_names[item.kind]);
  if (item.active) {
    ctx.fillStyle = item.active.warn ? "rgba(230, 58, 58, 0.66)" : "rgba(115, 230, 58, 0.66)";
    ctx.fillRect(-0.5, -0.5, 1, item.active.position);
  }
  ctx.restore();
}
function draw_player(player) {
  ctx.save();
  ctx.translate(player.anim_position.x, player.anim_position.y);
  ctx.rotate(-player.rot);
  if (player.boosting) ctx.scale(1.3, 1.3);
  draw_character(player.class, player.character);
  ctx.restore();
  ctx.save();
  for (const h of player.hands) {
    if (h.item) draw_item(h.item);
    ctx.translate(0.2, 0);
  }
  ctx.restore();
}
function draw_player_nametag(player) {
  if (overlay_vis_anim > 0.01) {
    ctx.save();
    ctx.translate(player.anim_position.x, player.anim_position.y);
    ctx.translate(0, -1);
    ctx.textAlign = "center";
    ctx.font = "15px sans-serif";
    ctx.scale(overlay_vis_anim / camera_scale, overlay_vis_anim / camera_scale);
    const w = ctx.measureText(player.name).width + 20;
    ctx.fillStyle = "#fffa";
    ctx.beginPath();
    ctx.roundRect(-w / 2, -10, w, 20, 5);
    ctx.fill();
    ctx.fillStyle = "black";
    ctx.textBaseline = "middle";
    ctx.fillText(player.name, 0, 0);
    ctx.restore();
  }
}
function draw_interact_target() {
  ctx.save();
  ctx.translate(interact_target_anim.x, interact_target_anim.y);
  ctx.lineCap = "round";
  ctx.lineJoin = "round";
  ctx.lineWidth = 0.06 + 0.03 * Math.sin(Date.now() / 100) * interact_possible_anim;
  ctx.strokeStyle = `hsla(${(1 - interact_active_anim) * 225 + interact_active_anim * 125}deg, ${interact_possible_anim * 100}%, 62.70%, ${interact_possible_anim * 0.7 + 0.3})`;
  ctx.strokeRect(0, 0, 1, 1);
  ctx.restore();
}
function draw_grid() {
  ctx.strokeStyle = "#333";
  ctx.lineWidth = 0.01;
  ctx.beginPath();
  const min = floor_v2(map_screen_to_world({ x: 0, y: 0 }));
  const max = ceil_v2(map_screen_to_world({ x: canvas.width, y: canvas.height }));
  for (let x = min.x; x < max.x; x++) {
    ctx.moveTo(x, min.y);
    ctx.lineTo(x, max.y);
  }
  for (let y = min.y; y < max.y; y++) {
    ctx.moveTo(min.x, y);
    ctx.lineTo(max.x, y);
  }
  ctx.stroke();
}
function draw_character(pclass, character) {
  ctx.fillStyle = `hsl(${character}rad, 50%, 50%)`;
  ctx.beginPath();
  ctx.arc(0, 0, PLAYER_SIZE, 0, Math.PI * 2);
  ctx.fill();
  if (pclass != "customer") {
    ctx.fillStyle = `hsl(${character}rad, 80%, 10%)`;
    ctx.beginPath();
    ctx.arc(0, -0.2, PLAYER_SIZE, 0, Math.PI * 2);
    ctx.fill();
  }
  if (pclass != "bot") {
    ctx.fillStyle = `hsl(${character}rad, 80%, 70%)`;
    ctx.beginPath();
    ctx.moveTo(-0.04, 0.25);
    ctx.lineTo(0.04, 0.25);
    ctx.lineTo(0, 0.4);
  }
  ctx.fill();
}
function message_str(m) {
  if ("text" in m) return m.text;
  if ("translation" in m) return tr(m.translation.id, ...m.translation.params.map(message_str));
  if ("tile" in m) return data.tile_names[m.tile];
  if ("item" in m) return data.item_names[m.item];
  return "[unknown message type]";
}
var MESSAGE_BG = { normal: "#fff", hint: "#111", error: "#fff", pinned: "rgb(4, 32, 0)" };
var MESSAGE_FG = { normal: "#000", hint: "#fff", error: "#a00", pinned: "#000" };
function draw_message(m, world) {
  ctx.save();
  ctx.translate(m.anim_position.x, m.anim_position.y);
  const scale = Math.min(m.anim_size, 1 - overlay_vis_anim);
  ctx.scale(scale, scale);
  if ("item" in m.inner) {
    ctx.fillStyle = MESSAGE_BG[m.style];
    if (m.style == "pinned") {
      ctx.translate(0, 1);
      ctx.beginPath();
      ctx.arc(0, -1, 0.5, 0, Math.PI * 2);
      ctx.fill();
    } else {
      ctx.beginPath();
      ctx.moveTo(0, -0.3);
      ctx.arc(0, -1, 0.5, Math.PI / 4, Math.PI - Math.PI / 4, true);
      ctx.closePath();
      ctx.fill();
    }
    if (m.timeout) {
      const t = m.timeout.remaining / m.timeout.initial;
      ctx.beginPath();
      ctx.strokeStyle = `hsl(${Math.sqrt(t) * 0.3}turn, 100%, 50%)`;
      ctx.lineWidth = 0.1;
      ctx.arc(0, -1, 0.45, -Math.PI / 2, -Math.PI / 2 + Math.PI * 2 * (1 - t));
      ctx.stroke();
    }
    ctx.translate(0, -1);
    draw_item_sprite(ctx, data.item_names[m.inner.item]);
    ctx.translate(0, 1);
  }
  if ("text" in m.inner || "translation" in m.inner) {
    ctx.translate(0, -1);
    ctx.textAlign = "center";
    ctx.font = "15px " + (world ? "sans-serif" : "monospace");
    if (world) ctx.scale(2 / camera_scale, 2 / camera_scale);
    const lines = message_str(m.inner).split("\n");
    const w = lines.reduce((a, v) => Math.max(a, ctx.measureText(v).width), 0) + 10;
    if (world) ctx.translate(0, -lines.length * 15 / 2);
    ctx.fillStyle = MESSAGE_BG[m.style];
    ctx.beginPath();
    ctx.roundRect(-w / 2, -5, w, lines.length * 15 + 10, 5);
    ctx.fill();
    ctx.fillStyle = MESSAGE_FG[m.style];
    ctx.textAlign = "left";
    ctx.textBaseline = "top";
    for (let i = 0; i < lines.length; i++)
      ctx.fillText(lines[i], -w / 2 + 5, i * 15);
    ctx.translate(0, 1);
  }
  ctx.restore();
}
function draw_global_message() {
  if (!global_message) return;
  ctx.save();
  ctx.translate(canvas.width / 2, canvas.height / 6);
  ctx.scale(2, 2);
  draw_message(global_message, false);
  ctx.restore();
}
function map_screen_to_world(screen) {
  return {
    x: (screen.x - canvas.width / 2) / camera_scale + camera.x,
    y: (screen.y - canvas.height / 2) / camera_scale + camera.y
  };
}

// main.ts
var KEY_INTERACT_LEFT = "KeyJ";
var KEY_INTERACT_RIGHT = "KeyL";
var KEY_BOOST = "KeyK";
var KEY_UP = "KeyW";
var KEY_DOWN = "KeyS";
var KEY_LEFT = "KeyA";
var KEY_RIGHT = "KeyD";
var KEY_CHAT = "Enter";
var KEY_CLOSE = "Escape";
var KEY_ZOOM = "KeyI";
var HANDLED_KEYS = [KEY_INTERACT_LEFT, KEY_BOOST, KEY_CHAT, KEY_CLOSE, KEY_DOWN, KEY_UP, KEY_LEFT, KEY_RIGHT];
var ctx;
var canvas;
var ws;
document.addEventListener("DOMContentLoaded", async () => {
  await init_locale();
  const ws_uri = globalThis.location.protocol.endsWith("s:") ? `wss://${globalThis.location.host}/` : `ws://${globalThis.location.hostname}:27032/`;
  ws = new WebSocket(ws_uri);
  ws.onerror = console.error;
  ws.onmessage = (m) => {
    packet(JSON.parse(m.data));
  };
  ws.onclose = () => console.log("close");
  ws.onopen = () => {
    console.log("open");
    send({ type: "join", name: "test", character: { color: Math.floor(Math.random() * 100), hairstyle: 0, headwear: 0 }, class: "chef" });
  };
  canvas = document.createElement("canvas");
  document.body.append(canvas);
  ctx = canvas.getContext("2d");
  resize();
  globalThis.addEventListener("resize", resize);
  draw();
  document.addEventListener("keydown", (ev) => keyboard(ev, true));
  document.addEventListener("keyup", (ev) => keyboard(ev, false));
  document.addEventListener("contextmenu", (ev) => ev.preventDefault());
  const tick_int = setInterval(tick_update, 1e3 / 25);
  ws.addEventListener("close", () => clearInterval(tick_int));
});
var players = /* @__PURE__ */ new Map();
var tiles = /* @__PURE__ */ new Map();
var items_removed = /* @__PURE__ */ new Set();
var server_hints = /* @__PURE__ */ new Map();
var data = { item_names: [], tile_names: [], spawn: [0, 0], tile_collide: [], tile_interact: [], maps: [], hand_count: 0 };
var global_message = void 0;
var my_id = -1;
var score = { active_recipes: 0, demands_completed: 0, demands_failed: 0, instant_recipes: 0, passive_recipes: 0, players: 0, points: 0, stars: 0, time_remaining: 0 };
var camera = { x: 0, y: 0 };
var camera_scale = 0.05;
var interact_target_anim = { x: 0, y: 0 };
var interact_possible_anim = 0;
var interact_active_anim = 0;
var overlay_vis_anim = 0;
var is_lobby = false;
var interacting;
var last_server_sent_position = { x: 0, y: 0 };
function get_item_location(loc) {
  if ("tile" in loc) return tiles.get(loc.tile.toString());
  if ("player" in loc) return players.get(loc.player[0]).hands[loc.player[1]];
  throw new Error("invalid item location");
}
function get_item_location_tracking(loc) {
  if ("tile" in loc) return tiles.get(loc.tile.toString()).position;
  if ("player" in loc) return players.get(loc.player[0]).position;
  throw new Error("invalid item location");
}
function send(p) {
  if (p.type != "movement") console.log("send", p);
  ws.send(JSON.stringify(p));
}
function packet(p) {
  if (!["movement", "update_map"].includes(p.type))
    console.log(p);
  switch (p.type) {
    case "version":
      console.log(`Protocol version: ${p.major}.${p.minor}`);
      break;
    case "joined":
      my_id = p.id;
      break;
    case "data":
      data = p.data;
      break;
    case "add_player": {
      players.set(p.id, {
        id: p.id,
        hands: new Array(data.hand_count).fill(null).map(() => ({})),
        position: { x: p.position[0], y: p.position[1] },
        anim_position: { x: p.position[0], y: p.position[1] },
        character: p.character.color,
        class: p.class,
        name: p.name,
        rot: 0,
        facing: { x: 0, y: 1 },
        vel: { x: 0, y: 0 },
        direction: { x: 0, y: 0 },
        stamina: 0,
        boosting: false
      });
      break;
    }
    case "remove_player":
      players.delete(p.id);
      break;
    case "movement": {
      const pl = players.get(p.player);
      const pos = { x: p.pos[0], y: p.pos[1] };
      if (p.player == my_id) return last_server_sent_position = pos;
      pl.position.x = pos.x;
      pl.position.y = pos.y;
      pl.boosting = p.boost;
      pl.rot = p.rot;
      break;
    }
    case "move_item": {
      const from = get_item_location(p.from);
      const to = get_item_location(p.to);
      to.item = from.item;
      to.item.tracking = get_item_location_tracking(p.to);
      from.item = void 0;
      break;
    }
    case "set_item": {
      const slot = get_item_location(p.location);
      const slotpos = get_item_location_tracking(p.location);
      if (slot.item !== void 0 && slot.item !== null) items_removed.add(slot.item);
      delete slot.item;
      if (p.item !== void 0 && p.item !== null) slot.item = { kind: p.item, x: slotpos.x, y: slotpos.y, tracking: slotpos };
      break;
    }
    case "clear_progress": {
      delete get_item_location(p.item).item.active;
      break;
    }
    case "set_progress": {
      const slot = get_item_location(p.item);
      if (!slot.item) return;
      slot.item.active = {
        position: p.position,
        speed: p.speed,
        warn: p.warn
      };
      break;
    }
    case "update_map":
      if (p.kind !== void 0 && p.kind !== null)
        tiles.set(p.tile.toString(), {
          x: p.tile[0],
          y: p.tile[1],
          position: { x: p.tile[0] + 0.5, y: p.tile[1] + 0.5 },
          kind: p.kind
        });
      else
        tiles.delete(p.tile.toString());
      break;
    case "communicate": {
      const player = players.get(p.player);
      if (p.message) {
        const message = {
          inner: p.message,
          anim_size: player.message_persist?.anim_size ?? 0,
          anim_position: player.anim_position,
          timeout: p.timeout ?? { initial: 5, remaining: 5, pinned: false },
          style: "normal"
        };
        if (p.timeout) player.message_persist = message;
        else player.message = message;
        if (p.timeout?.pinned) player.message_pinned = {
          inner: p.message,
          anim_size: player.message_pinned?.anim_size ?? 0,
          anim_position: { x: 20, y: 0 },
          style: "pinned",
          timeout: p.timeout
        };
      } else {
        delete player.message_persist;
        delete player.message_pinned;
      }
      break;
    }
    case "score":
      score.demands_completed = p.demands_completed;
      score.demands_failed = p.demands_failed;
      score.points = p.points;
      score.time_remaining = p.time_remaining ?? null;
      break;
    case "server_message":
      global_message = {
        inner: p.message,
        style: p.error ? "error" : "normal",
        anim_size: 0,
        anim_position: { x: 0, y: 0 },
        timeout: { initial: 5, remaining: 5, pinned: false }
      };
      break;
    case "set_ingame":
      console.log(`ingame ${p.state}`);
      is_lobby = p.lobby;
      break;
    case "movement_sync": {
      const me = players.get(my_id);
      if (me) {
        me.position.x = last_server_sent_position.x;
        me.position.y = last_server_sent_position.y;
      }
      break;
    }
    case "server_hint":
      if (p.player != my_id) return;
      if (p.message) server_hints.set(p.position + "", {
        inner: p.message,
        anim_size: 0,
        anim_position: p.position ? { x: p.position[0] + 0.5, y: p.position[1] + 0.5 } : players.get(my_id).anim_position,
        style: "hint"
      });
      else server_hints.delete(p.position + "");
      break;
    case "tutorial_ended":
      break;
    case "environment":
      break;
    case "effect":
      break;
    case "flush_map":
      break;
    case "menu":
      switch (p.menu) {
        case "document":
          if (JSON.stringify(p.data).search("b.toc") != -1)
            open("https://s.metamuffin.org/static/hurrycurry/book.pdf");
          else alert("document display not supported yet");
          break;
        case "score":
          global_message = {
            timeout: { initial: 5, remaining: 5, pinned: false },
            inner: { text: `Score: ${JSON.stringify(p.data, null, 4)}` },
            anim_position: { x: 0, y: 0 },
            anim_size: 0,
            style: "normal"
          };
          break;
        default:
          console.warn("unknown menu");
      }
      break;
    default:
      console.warn("unknown packet", p);
  }
}
var chat = null;
var QUICK_COMMANDS = {
  "Numpad1": "/start junior",
  "Numpad2": "/start debug2",
  "Numpad3": "/start sophomore",
  "Numpad4": "/start debug",
  "Numpad5": "/start 5star",
  "Numpad6": "/start campaign/lobby",
  "Numpad8": "/start-tutorial plate:seared-patty,sliced-bun",
  "Numpad9": "/start-tutorial plate:bun",
  "Numpad7": "/end-tutorial",
  "Numpad0": "/end",
  "NumpadEnter": "/r"
};
var keys_down = /* @__PURE__ */ new Set();
function keyboard(ev, down) {
  if (down && ev.code == KEY_CHAT) return toggle_chat();
  else if (down && ev.code == KEY_CLOSE && chat) return close_chat();
  else if (chat) return;
  if (HANDLED_KEYS.includes(ev.code)) ev.preventDefault();
  if (!keys_down.has(KEY_INTERACT_LEFT) && ev.code == KEY_INTERACT_LEFT && down) set_interact(true, 0);
  if (keys_down.has(KEY_INTERACT_LEFT) && ev.code == KEY_INTERACT_LEFT && !down) set_interact(false, 0);
  if (!keys_down.has(KEY_INTERACT_RIGHT) && ev.code == KEY_INTERACT_RIGHT && down) set_interact(true, 1);
  if (keys_down.has(KEY_INTERACT_RIGHT) && ev.code == KEY_INTERACT_RIGHT && !down) set_interact(false, 1);
  if (down && ev.code in QUICK_COMMANDS) send({ player: my_id, type: "communicate", message: { text: QUICK_COMMANDS[ev.code] } });
  if (down && ev.code == "KeyE") particle_splash(get_interact_target() ?? { x: 0, y: 0 }, 0.8);
  if (down) keys_down.add(ev.code);
  else keys_down.delete(ev.code);
}
function close_chat() {
  if (!chat) return;
  chat.remove();
  canvas.focus();
  chat = null;
}
function toggle_chat() {
  if (chat) {
    if (chat.value.length) send({ player: my_id, type: "communicate", message: { text: chat.value } });
    chat.remove();
    canvas.focus();
    chat = null;
  } else {
    chat = document.createElement("input");
    chat.type = "text";
    chat.placeholder = "Message";
    document.body.append(chat);
    chat.focus();
  }
}
function get_interact_target() {
  if (interacting) return interacting;
  const me = players.get(my_id);
  if (!me) return;
  if (location.hash.search("newinteract") == -1)
    return {
      x: Math.floor(me.position.x + Math.sin(me.rot)),
      y: Math.floor(me.position.y + Math.cos(me.rot))
    };
  const rx = me.position.x + Math.sin(me.rot) * 0.7;
  const ry = me.position.y + Math.cos(me.rot) * 0.7;
  const bx = Math.floor(rx);
  const by = Math.floor(ry);
  let best = { x: bx, y: by };
  let best_d = 100;
  for (let ox = -1; ox <= 1; ox++) for (let oy = -1; oy <= 1; oy++) {
    const t = tiles.get([bx + ox, by + oy] + "");
    if (!t) continue;
    if (data.tile_interact[t.kind] || t.item) {
      const cx = bx + ox + 0.5;
      const cy = by + oy + 0.5;
      const dx = rx - cx;
      const dy = ry - cy;
      const pdx = me.position.x - cx;
      const pdy = me.position.y - cy;
      const d = Math.sqrt(dx * dx + dy * dy);
      const pd = Math.sqrt(pdx * pdx + pdy * pdy);
      if (pd < 2 && d < best_d) {
        best_d = d;
        best = { x: bx + ox, y: by + oy };
      }
    }
  }
  return best;
}
function set_interact(edge, hand) {
  if (edge) interacting = get_interact_target();
  if (interacting) send({ player: my_id, type: "interact", pos: edge ? [interacting.x, interacting.y] : void 0, hand });
  if (!edge) interacting = void 0;
}
function tick_update() {
  const p = players.get(my_id);
  if (!p) return;
  send({ player: my_id, type: "movement", pos: [p.position.x, p.position.y], dir: [p.direction.x, p.direction.y], boost: p.boosting });
}
function frame_update(dt) {
  const p = players.get(my_id);
  if (!p) return;
  score.time_remaining -= dt;
  const direction = normalize({
    x: +keys_down.has(KEY_RIGHT) - +keys_down.has(KEY_LEFT),
    y: +keys_down.has(KEY_DOWN) - +keys_down.has(KEY_UP)
  });
  if (interacting) direction.x *= 0, direction.y *= 0;
  p.direction = direction;
  update_movement(p, dt, direction, keys_down.has("KeyK"));
  for (const [_, a] of players)
    for (const [_2, b] of players)
      collide_player_player(a, b, dt);
  const update_item = (item) => {
    if (item.tracking) lerp_exp_v2_mut(item, item.tracking, dt * 10);
    if (item.active) item.active.position += item.active.speed * dt;
  };
  let pin_xo = 0;
  for (const [pid, player] of players) {
    if (pid == my_id) player.anim_position.x = player.position.x, player.anim_position.y = player.position.y;
    else lerp_exp_v2_mut(player.anim_position, player.position, dt * 15);
    player.hands.forEach((h) => {
      if (h.item) update_item(h.item);
    });
    if (player.message && tick_message(player.message, dt)) delete player.message;
    if (player.message_persist && tick_message(player.message_persist, dt)) delete player.message_persist;
    if (player.message_pinned && tick_message(player.message_pinned, dt)) delete player.message_pinned;
    if (player.message_pinned) lerp_exp_v2_mut(player.message_pinned.anim_position, { x: pin_xo++, y: 0 }, dt * 5);
  }
  for (const [_, tile] of tiles)
    if (tile.item !== void 0 && tile.item !== null) update_item(tile.item);
  const remove = [];
  for (const item of items_removed) {
    update_item(item);
    if (item.remove_anim === void 0) item.remove_anim = 0;
    item.remove_anim += dt * 4;
    if (item.remove_anim > 1) remove.push(item);
  }
  remove.forEach((i) => items_removed.delete(i));
  for (const [_, h] of server_hints)
    tick_message(h, dt);
  lerp_exp_v2_mut(camera, p.position, dt * 10);
  if (global_message && tick_message(global_message, dt)) global_message = void 0;
  const it = get_interact_target() ?? { x: 0, y: 0 };
  const possible = data.tile_interact[tiles.get([it.x, it.y].toString())?.kind ?? 0] ?? false;
  lerp_exp_v2_mut(interact_target_anim, it, dt * 15);
  interact_possible_anim = lerp_exp(interact_possible_anim, +possible, dt * 18);
  interact_active_anim = lerp_exp(interact_active_anim, +!!interacting, dt * 15);
  const zoom_target = Math.min(canvas.width, canvas.height) * (keys_down.has(KEY_ZOOM) ? 0.05 : 0.1);
  camera_scale = lerp_exp(camera_scale, zoom_target, dt * 5);
  overlay_vis_anim = lerp_exp(overlay_vis_anim, +keys_down.has(KEY_ZOOM), dt * 10);
  tick_particles(dt);
}
function resize() {
  canvas.width = globalThis.innerWidth;
  canvas.height = globalThis.innerHeight;
}
var last_frame = performance.now();
function draw() {
  const now = performance.now();
  frame_update((now - last_frame) / 1e3);
  last_frame = now;
  if (ws.readyState == ws.CONNECTING) draw_wait("Connecting...");
  else if (ws.readyState == ws.CLOSING) draw_wait("Closing...");
  else if (ws.readyState == ws.CLOSED) draw_wait("Disconnected");
  else if (ws.readyState == ws.OPEN) draw_ingame();
  else throw new Error(`ws state invalid`);
  requestAnimationFrame(draw);
}
function tick_message(m, dt) {
  if (!m) return true;
  m.anim_size = lerp_exp(m.anim_size, m.timeout ? m.timeout.remaining > 0.3 ? 1 : 0 : 1, dt * 3);
  if (!m.timeout) return false;
  m.timeout.remaining -= dt;
  return m.timeout.remaining <= 0;
}
export {
  camera,
  camera_scale,
  canvas,
  chat,
  ctx,
  data,
  get_interact_target,
  global_message,
  interact_active_anim,
  interact_possible_anim,
  interact_target_anim,
  is_lobby,
  items_removed,
  keys_down,
  my_id,
  overlay_vis_anim,
  players,
  score,
  server_hints,
  tiles
};
